@@ -42,8 +42,8 @@ sub spawn {
   my $self = { %args };
   bless $self,$class;
 
-  local $ENV{TEST_HTTP_VERBOSE} = 1
-    if (delete $args{debug});
+  local $ENV{TEST_HTTP_VERBOSE};
+  $ENV{TEST_HTTP_VERBOSE} = 1 if delete $args{debug};
 
   $self->{delete} = [];
   if (my $html = delete $args{html}) {
@@ -81,7 +81,7 @@ sub spawn {
   $lhurl->host( "localhost" );
   $self->{_server_url} = $lhurl;
 
-  diag "Started local server on $lhurl";
+  diag "Started $lhurl";
 
   $self;
 };
@@ -144,14 +144,19 @@ sub get_output {
   open LOG, "<", $self->{logfile}
     or die "Couldn't retrieve logfile";
   join "", <LOG>;
-};
+}
 
 sub DESTROY {
-  $_[0]->stop if $_[0]->{_server_url};
-  for my $file (@{$_[0]->{delete}}) {
-    unlink $file or warn "Couldn't remove tempfile $file : $!\n";
-  };
-};
+    my $self = shift;
+    $self->stop if $self->{_server_url};
+    if ( $self->{_fh} ) {
+        close $self->{_fh};
+        delete $self->{_fh};
+    }
+    for my $file ( @{$self->{delete}} ) {
+        unlink $file or warn "Couldn't remove tempfile $file : $!\n";
+    }
+}
 
 =head1 EXPORT
 
